#!/bin/sh
_http_hdr_add_ct() {
	local ct="$1"
	[ -z "${ct}" ] && ct="text/html"
	echo -ne "Content-Type: ${ct}\r\n"
	return 0
}
_http_hdr_add_sep() {
	echo -ne "\r\n"
	return 0
}
_http_hdr_file_dl() {
	local size="${1}"
	local name="${2}"
	_http_hdr_add_ct "application/octet-stream"
	echo -ne "Content-Length: ${size}\r\n"
	echo -ne "Content-Transfer-Encoding: binary\r\n"
	echo -ne "Content-Disposition: attachment; filename=${name}\r\n"
	_http_hdr_add_sep
	return 0
}
http_hdr_ct_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	local ct=""
	utils_check_retv "${retv}" "rslt" "ct" || return $?
	if [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	haserl_var_get "CONTENT_TYPE" ct || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' retrieving Content-Type!"
		return ${ret}
	fi
	IFS=';' read -r rslt dummy <<< "${ct}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' in 'read' command!"
		return ${ret}
	fi
	eval "${retv}=\${rslt}"
	return 0
}
http_hdr_ubus() {
	[ ${BE_TGT} -eq 0 ] && return 0
	_http_hdr_add_ct "application/json"
	_http_hdr_add_sep
	return 0
}
http_file_dl() {
	local path="${1}"
	local name="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local size
	utils_cmd_exists "wc" || return $?
	if [ -z "${path}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	size=$(wc -c 2>&1 < "${path}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' calculating size of '${path}'!"
		jlog_err ${f} "${size}"
		return ${ret}
	fi
	_http_hdr_file_dl "${size}" "${name}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding HTTP Headers!"
		return ${ret}
	fi
	cat "${path}" 2>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' echoing '${path}' through 'cat'!"
		return ${ret}
	fi
	return 0
}
INC_HTTP=1
