#!/bin/sh
INSTRM_UCI="instreamer4"
_instrm_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local udata
	local jdata
	utils_check_retv "${retv}" "uci" "udata" "jdata" || return $?
	utils_cmd_exists "awk" || return $?
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	udata=$(${uci} export "${INSTRM_UCI}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error exporting UCI data from '"${INSTRM_UCI}"'!"
		jlog_err ${f} "${udata}"
		return ${ret}
	fi
	jdata=$(awk -f ${AWK}/uci-to-json.awk <<< "${udata}" 2>&1) \
		|| ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not translate UCI into JSON!"
		jlog_err ${f} "${jdata}"
		return ${ret}
	fi
	eval "${retv}=\${jdata}"
	return 0
}
_instrm_set() {
	local data="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local json_file=""
	local uci_file=""
	utils_cmd_exists "awk" || return $?
	utils_cmd_exists "cat" || return $?
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	utils_mktemp_file jfile || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not create temporary file for JSON data!"
		return ${ret}
	fi
	utils_mktemp_file ufile || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not create temporary file for UCI commands!"
		return ${ret}
	fi
	echo "${data}" > "${jfile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error copying data to '"${jfile}"'!"
		return ${ret}
	fi
	awk -v UCI_PKG="${INSTRM_UCI}" -f ${AWK}/json-to-uci.awk "${jfile}" > "${ufile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not translate JSON into UCI!"
		return ${ret}
	fi
	${uci} -f "${ufile}" batch || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error in 'uci batch' command on '"${ufile}"'!"
		return ${ret}
	fi
	if [ ${BE_TGT} -eq 1 ]; then
		[ -z "${INC_FUNCS}" ] && . ${BRX_UCI}/functions.sh && INC_FUNCS=1
		cfg_save || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Error applying configuration through 'cfg_save'!"
			return ${ret}
		fi
	else
		${uci} commit || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Error in '${uci} commit'!"
			return ${ret}
		fi
	fi
	utils_rm_file "${jfile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error removing '"${jfile}"'!"
		return ${ret}
	fi
	utils_rm_file "${ufile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error removing '"${ufile}"'!"
		return ${ret}
	fi
	return 0
}
_instrm_disable() {
	local f="${FUNCNAME[0]}"
	local ret=0
	app_stop "audiostreamer" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not stop 'audiostreamer'!"
		return ${ret}
	fi
	return 0
}
_instrm_enable() {
	local f="${FUNCNAME[0]}"
	local ret=0
	app_start "audiostreamer" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not start 'audiostreamer'!"
		return ${ret}
	fi
	return 0
}
_instrm_status() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local json
	utils_check_retv "${retv}" "json" || return $?
	json_init
	app_is_running "audiostreamer"
	rslt=$?
	if [ ${rslt} -eq 0 ]; then
		json_add_string "streaming" "enabled"
		slog_inf ${f} "Streaming is enabled"
	elif [ ${rslt} -eq 1 ]; then
		json_add_string "streaming" "disabled"
		slog_inf ${f} "Streaming is disabled"
	else
		jlog_err ${f} "Could not check 'audiostreamer' status!"
		return ${rslt}
	fi
	json=$(json_dump)
	json_cleanup
	eval "${retv}=\${json}"
	return 0
}
_instrm_import() {
	local sign="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local fpath=""
	local fname=""
	local uci=""
	jq_str_get "${sign}" "filepath" fpath || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get the file path from 'signature'!"
		return ${ret}
	fi
	jq_str_get "${sign}" "filename" fname || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get the file name from 'signature'!"
		return ${ret}
	fi
	cat /dev/null > "${BE_UCI}/${INSTRM_UCI}"
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	${uci} -f "${fpath}" -m import "${INSTRM_UCI}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' in 'uci import' command on '"${fpath}"'!"
		return ${ret}
	fi
	[ -z "${INC_UCI_CMT}" ] && . ${SH}/be_uci_commit.sh
	sign='{"config": "'${INSTRM_UCI}'"}'
	uci_commit "${sign}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error committing changes on '"${fpath}"'!"
		return ${ret}
	fi
	return 0
}
_instrm_export() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local udata
	local tfile=""
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' getting UCI tool's command line!"
		return ${ret}
	fi
	udata=$(${uci} export "${INSTRM_UCI}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' exporting UCI data from '${INSTRM_UCI}'!"
		jlog_err ${f} "${udata}"
		return ${ret}
	fi
	utils_mktemp_file tfile || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' creating a temporary file!"
		return ${ret}
	fi
	echo "${udata}" > "${tfile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' copying data to '${tfile}'!"
		utils_rm_file "${tfile}"
		return ${ret}
	fi
	http_file_dl "${tfile}" "${INSTRM_UCI}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' sending file through HTTP!"
		utils_rm_file "${tfile}"
		return ${ret}
	fi
	utils_rm_file "${tfile}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' removing '${tfile}'!"
		return ${ret}
	fi
	# This request is a dead end! Log what has been done and exit. It is not
	slog_inf ${f} "Exported '${tfile}' as '${INSTRM_UCI}' with UCI configuration"
	exit 0
}
_instrm_restore() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local file
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not get UCI tool's command line!"
		return ${ret}
	fi
	file="${BE_UCI_DEF}/${INSTRM_UCI}"
	if [ ! -f "${file}" ]; then
		jlog_err "${f}" "File '${file}' does not exist!"
		return 1
	fi
	${uci} delete "${INSTRM_UCI}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' in 'uci delete ${INSTRM_UCI}'!"
		return ${ret}
	fi
	${uci} -f "${file}" -m import "${INSTRM_UCI}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' in 'uci import' from '${file}'!"
		${uci} revert "${INSTRM_UCI}" >/dev/null 2>&1 || \
			jlog_err "${f}" "Could not revert changes performed!"
		return ${ret}
	fi
	[ -z "${INC_UCI_CMT}" ] && . ${SH}/be_uci_commit.sh
	sign='{"config": "'${INSTRM_UCI}'"}'
	uci_commit "${sign}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error committing changes on '${INSTRM_UCI}'!"
		${uci} revert "${INSTRM_UCI}" >/dev/null 2>&1 || \
			jlog_err "${f}" "Could not revert changes performed!"
		return ${ret}
	fi
	return 0
}
_instrm_remove() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not get UCI tool's command line!"
		return ${ret}
	fi
	app_stop "audiostreamer" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not stop 'audiostreamer'!"
		return ${ret}
	fi
	${uci} delete "${INSTRM_UCI}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' in 'uci delete ${INSTRM_UCI}'!"
		return ${ret}
	fi
	${uci} commit "${INSTRM_UCI}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' in 'uci commit ${INSTRM_UCI}'!"
		return ${ret}
	fi
	return 0
}
do_instrm() {
	local proc="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local sign=""
	local ires=""
	utils_check_retv "${retv}" "proc" "sign" "ires" || return $?
	if [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	case "${proc}" in
	"get")
		_instrm_get ires || ret=$?
		;;
	"set")
		ubus_sign_get "${post_body}" sign || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get signature of POST request!"
			break
		fi
		_instrm_set "${sign}" || ret=$?
		;;
	"disable")
		_instrm_disable || ret=$?
		;;
	"enable")
		_instrm_enable || ret=$?
		;;
	"status")
		_instrm_status ires || ret=$?
		;;
	"import")
		ubus_sign_get "${post_body}" sign || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get signature of POST request!"
			break
		fi
		_instrm_import "${sign}" || ret=$?
		;;
	"export")
		_instrm_export || ret=$?
		;;
	"restore")
		_instrm_restore || ret=$?
		;;
	"remove")
		_instrm_remove || ret=$?
		;;
	*)
		jlog_err ${f} "Procedure '${proc}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error occurred in '${proc}' procedure!"
		return ${ret}
	fi
	eval "${retv}=\${ires}"
	return 0
}
INC_INSTRM=1
