#!/bin/sh
LOGIN_NONCE_PATH="${PASSWD}/nonce"
LOGIN_PASSWD_PATH="${PASSWD}/passwd.hash"
readonly LOGIN_PASSWD_OK='{"status":"200 OK"}'
readonly LOGIN_PASSWD_KO='{"status":"401 Access denied"}'
_login_conf_host() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local tmp="/tmp/instreamer-ice"
	LOGIN_NONCE_PATH="${tmp}/nonce"
	LOGIN_PASSWD_PATH="${tmp}/passwd"
	mkdir -p "${tmp}" &>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not create '${tmp}'!"
	fi
	slog_inf "${f}" "Created '${tmp}' directory"
	return ${ret}
}
_login_nonce_reset() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local nonce
	local json
	utils_check_retv "${retv}" "nonce" "json" || return $?
	utils_cmd_exists "date" || return $?
	utils_cmd_exists "md5sum" || return $?
	utils_cmd_exists "cut" || return $?
	nonce=$(date | md5sum | cut -d' ' -f1)
	echo "${nonce}" 2>/dev/null > "${LOGIN_NONCE_PATH}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not store 'nonce' in '${LOGIN_NONCE_PATH}'!"
		return ${ret}
	fi
	json='{"nonce":"'${nonce}'"}'
	eval "${retv}=\${json}"
	return 0
}
_login_passwd_check() {
	local json_auth="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local user_i=""
	local hash_i=""
	local nonce
	local passwd
	local hash_u
	local rehash
	local json=""
	utils_check_retv "${retv}" "json_auth" "user_i" "hash_i" "nonce" \
		"passwd" "hash_u" "rehash" "json" || return $?
	utils_cmd_exists "cat" || return $?
	utils_cmd_exists "md5sum" || return $?
	utils_cmd_exists "cut" || return $?
	jq_str_get "${json_auth}" "user" user_i || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not get user from authentication data!"
		return ${ret}
	fi
	jq_str_get "${json_auth}" "hash" hash_i || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not get hash from authentication data!"
		return ${ret}
	fi
	nonce=$(cat "${LOGIN_NONCE_PATH}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not read '${LOGIN_NONCE_PATH}'!"
		jlog_err "${f}" "${nonce}"
		return ${ret}
	fi
	passwd=$(cat "${LOGIN_PASSWD_PATH}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not read '${LOGIN_PASSWD_PATH}'!"
		jlog_err "${f}" "${passwd}"
		return ${ret}
	fi
	hash_u=$(grep "${user_i}" <<< "${passwd}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Could not get hash for '${user_i}'!"
		return ${ret}
	fi
	rehash=$(echo -n "${hash_u}:${nonce}" | md5sum | cut -d' ' -f1)
	if [ "${hash_i}" =  "${rehash}" ] ; then
		_login_nonce_reset dummy
		json=${LOGIN_PASSWD_OK}
	else
		json=${LOGIN_PASSWD_KO}
	fi
	eval "${retv}=\${json}"
	return 0
}
_login_passwd_change() {
	local jauth="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local user=""
	local chash=""
	local nhash=""
	local lsign
	local rslt=""
	utils_check_retv "${retv}" "jauth" "user" "chash" "nhash" "lsign" \
		"rslt" || return $?
	jq_str_get "${jauth}" "user" user || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' getting 'user' from authentication data!"
		return ${ret}
	fi
	jq_str_get "${jauth}" "hash" chash || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' getting 'hash' from authentication data!"
		return ${ret}
	fi
	jq_str_get "${jauth}" "nhash" nhash || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' getting 'nhash' from authentication data!"
		return ${ret}
	fi
	lsign='{"user":"'${user}'","hash":"'${chash}'"}'
	_login_passwd_check "${lsign}" rslt || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' checking password!"
		return ${ret}
	fi
	if [ "${rslt}" = "${LOGIN_PASSWD_KO}" ] ; then
		eval "${retv}=\${LOGIN_PASSWD_KO}"
		return 0
	fi
	regex="s/(^${user}:[^:]*).*/\1:${nhash}/g"
	sed -E -e "${regex}" -i "${LOGIN_PASSWD_PATH}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error '${ret}' replacing '${user}' password on '${LOGIN_PASSWD_PATH}'!"
		return ${ret}
	fi
	eval "${retv}=\${LOGIN_PASSWD_OK}"
	return 0
}
do_login() {
	local proc="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local sign=""
	local lres=""
	utils_check_retv "${retv}" "proc" "sign" "lres" || return $?
	if [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err "${f}" "Missing mandatory arguments!"
		return 1
	fi
	# TEST MODE! When testing the Backend Shell Service handler for Login on
	if [ "${BE_TGT}" -eq 0 ]; then
		_login_conf_host || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err "${f}" "Could not configure host for testing!"
			return ${ret}
		fi
	fi
	case "${proc}" in
	"nonce_get")
		_login_nonce_reset lres || ret=$?
		;;
	"passwd_check")
		ubus_sign_get "${post_body}" sign || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err "${f}" "Could not get signature of POST request!"
		else
			_login_passwd_check "${sign}" lres || ret=$?
		fi
		;;
	"passwd_change")
		ubus_sign_get "${post_body}" sign || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err "${f}" "Could not get signature of POST request!"
		else
			_login_passwd_change "${sign}" lres || ret=$?
		fi
		;;
	*)
		jlog_err "${f}" "Procedure '${proc}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Error occurred in '${proc}' procedure!"
		return ${ret}
	fi
	eval "${retv}=\${lres}"
	return 0
}
INC_LOGIN=1
