#!/bin/sh
_sys_factory_defaults() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local rslt
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	rslt=$(${uci} get "security.defaults.enabled" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing '${uci} get security.defaults.enabled 2>&1'!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	if [ "${rslt}" != "true" ]; then
		jlog_err ${f} "Restore factory defaults is not allowed!"
		return 1
	fi
	[ -z "${INC_FUNCS}" ] && . ${BRX_UCI}/functions.sh && INC_FUNCS=1
	cfg_restore_defaults || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'cfg_restore_defaults'!"
		return ${ret}
	fi
	return 0
}
do_sys() {
	local proc="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local sres=""
	utils_check_retv "${retv}" "proc" "sres" || return $?
	if [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	case "${proc}" in
	"factory_defaults")
		_sys_factory_defaults sres || ret=$?
		;;
	*)
		jlog_err ${f} "Procedure '${proc}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error occurred in '${proc}' procedure!"
		return ${ret}
	fi
	eval "${retv}=\${sres}"
	return 0
}
INC_SYS=1
