#!/bin/sh
_uci_commit_host() {
	local pkg="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci=""
	local rslt
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	rslt=$(${uci} commit ${pkg} 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' in '${uci} commit ${pkg}'!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	return 0
}
_uci_commit_target() {
	local f="${FUNCNAME[0]}"
	local ret=0
	[ -z "${INC_FUNCS}" ] && . ${BRX_UCI}/functions.sh && INC_FUNCS=1
	cfg_save || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'cfg_save'!"
		return ${ret}
	fi
	return 0
}
_uci_commit() {
	local pkg="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	if [ ${BE_TGT} -eq 1 ]; then
		_uci_commit_target || ret=$?
	else
		_uci_commit_host "${pkg}" || ret=$?
	fi
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not commit UCI changes'!"
		return ${ret}
	fi
	return 0
}
_uci_commit_load() {
	local sign="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local keys=""
	local key
	local blob=""
	utils_check_retv "${retv}" "sign" "keys" "key" "blob" || return $?
	jq_keys_get "${sign}" keys || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get signature's keys!"
		return ${ret}
	fi
	for key in ${keys}; do
		case "${key}" in
		"config")
			jq_str_get "${sign}" "${key}" blob || ret=$?
			if [ ${ret} -ne 0 ]; then
				jlog_err ${f} "Could not get 'config' value!"
				return ${ret}
			fi
			;;
		*)
			jlog_err ${f} "Key '${key}' is invalid!"
			return 1
			;;
		esac
	done
	eval "${retv}=\${blob}"
	return 0
}
uci_commit() {
	local sign="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local cblob=""
	if [ -z "${sign}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_uci_commit_load "${sign}" cblob || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error loading 'signature'!"
		return ${ret}
	fi
	_uci_commit "${cblob}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error committing UCI configuration!"
		return ${ret}
	fi
	return 0
}
INC_UCI_CMT=1
