#!/bin/sh
_uci_get_add_val() {
	local name="${1}"
	local vals="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local idx=0
	local val
	local val_qt
	eval "set -- ${vals}"
	idx=0
	for val; do
		idx=$((${idx} + 1))
	done
	if [ ${idx} -le 1 ]; then
		val_qt=$(utils_rem_squote "${vals}") || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not remove single quotes on ${vals} option!"
			return ${ret}
		fi
		json_add_string "${name}" "${val_qt}"
		return 0
	fi
	json_add_array "${name}"
	eval "set -- ${vals}"
	for val; do
		val_qt=$(utils_rem_squote "${val}") || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not remove single quotes on '${val}' option!"
			return ${ret}
		fi
		json_add_string "" "${val_qt}"
	done
	json_close_array
	return 0
}
_uci_get_add_opt() {
	local pkg="${1}"
	local sec="${2}"
	local opt="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local path
	local uci=""
	local rslt
	local vals
	utils_cmd_exists "sed" || return $?
	path="${pkg}.${sec}.${opt}"
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	rslt=$(${uci} show "${path}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing '${uci} show ${path} 2>&1'!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	vals=$(sed -e "s/${path}=//g" 2>&1 <<< "${rslt}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get values for '${path}'!"
		return ${ret}
	fi
	if [ -z "${vals}" ]; then
		jlog_err ${f} "Value for '${path}' is empty!"
		return 1
	fi
	_uci_get_add_val "value" "${vals}" || ret=$?
	return ${ret}
}
_uci_get_add_sec() {
	local pkg="${1}"
	local sec="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local path
	local uci=""
	local rslt
	local line
	local vals
	local key val
	utils_cmd_exists "sed" || return $?
	path="${pkg}.${sec}"
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	rslt=$(${uci} show "${path}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing '${uci} show ${path} 2>&1'!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	json_add_object "values"
	json_add_boolean ".anonymous" 0
	while IFS= read -r line; do
		vals=$(sed -e "s/${path}//g" 2>&1 <<< "${line}") || return 1
		while IFS='=' read -r key val; do
			if [ -z "${key}" ]; then
				json_add_string ".type" "${val}"
				json_add_string ".name" "${sec}"
			else
				key=$(sed -e "s/^\.//g" 2>&1 <<< "${key}") \
					|| return 2
				_uci_get_add_val "${key}" "${val}" || return $?
			fi
		done <<< "${vals}"
	done <<< "${rslt}"
	json_close_object
	return 0
}
_uci_get_add_pkg() {
	local pkg="${1}"
	local typ="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local path
	local uci=""
	local rslt
	local sec_idx
	local sec_allw
	local sec_opn
	local line
	local upkg usec utyp
	local key val
	path="${pkg}"
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	rslt=$(${uci} show "${path}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing '${uci} show ${path} 2>&1'!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	json_add_object "values"
	sec_idx=0
	sec_allw=1
	[ -n "${typ}" ] && sec_allw=0
	sec_opn=0
	while IFS= read -r line; do
		IFS=".=" read -r upkg usec utyp <<< "${line}"
		if [ "${utyp#*=}" = "${utyp}" ]; then
			if [ -n "${typ}" ];  then
				if [ "${typ}" = "${utyp}" ]; then
					sec_allw=1
				else
					sec_allw=0
				fi
			fi
			if [ ${sec_allw} -eq 0 ]; then
				sec_idx=$((${sec_idx} + 1))
				continue
			fi
			if [ ${sec_opn} -eq 1 ]; then
				json_close_object
				sec_opn=0
			fi
			json_add_object "${usec}"
			sec_opn=1
			if [ "@${utyp}" = "${usec%[*}" ]; then
				json_add_boolean ".anonymous" 1
			else
				json_add_boolean ".anonymous" 0
			fi
			json_add_string ".type" "${utyp}"
			json_add_string ".name" "${usec}"
			json_add_int ".index" "${sec_idx}"
			sec_idx=$((${sec_idx} + 1))
		else
			[ ${sec_allw} -eq 0 ] && continue
			while IFS='=' read -r key val; do
				_uci_get_add_val "${key}" "${val}" || return $?
			done <<< "${utyp}"
		fi
	done <<< "${rslt}"
	json_close_object
	return 0
}
_uci_get_reply() {
	local blob="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local pkg sec typ opt
	local pkg_list
	local upkg
	utils_cmd_exists "ls" || return $?
	IFS=":" read -r pkg sec typ opt <<< "${blob}" || return 1
	if [ -n "${opt}" ]; then
		_uci_get_add_opt "${pkg}" "${sec}" "${opt}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not add '${pkg}.${sec}.${opt}' option!"
			return ${ret}
		fi
	elif [ -n "${sec}" ]; then
		_uci_get_add_sec "${pkg}" "${sec}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not add '${pkg}.${sec}' section!"
			return ${ret}
		fi
	elif [ -n "${pkg}" ]; then
		_uci_get_add_pkg "${pkg}" "${typ}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not add '${pkg}' package!"
			return ${ret}
		fi
	elif [ -z "${pkg}" ]; then
		pkg_list=$(ls "${BE_UCI}" 2>&1) || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not get list of packages!"
			jlog_err ${f} "${pkg_list}"
			return ${ret}
		fi
		json_add_array "packages"
		for upkg in ${pkg_list}; do
			json_add_string "" "${upkg}"
		done
		json_close_array
	fi
	return ${ret}
}
_uci_get_validate() {
	local blob="${1}"
	local f="${FUNCNAME[0]}"
	local pkg sec typ opt
	IFS=":" read -r pkg sec typ opt <<< "${blob}" || return 1
	if [ -z "${pkg}" ]; then
		# No 'package', no nothing else!
		if [ -n "${sec}" ]; then
			jlog_err ${f} "Section cannot be specified!"
			return 2
		fi
		if [ -n "${opt}" ]; then
			jlog_err ${f} "Option cannot be specified!"
			return 3
		fi
		if [ -n "${typ}" ]; then
			jlog_err ${f} "Type cannot be specified!"
			return 4
		fi
	elif [ -n "${opt}" ]; then
		# For 'option', can't have both 'section' and 'type', nor none!
		if [ -z "${sec}" ] && [ -z "${typ}" ]; then
			jlog_err ${f} "Section or Type must be specified!"
			return 5
		elif [ -n "${sec}" ] && [ -n "${typ}" ]; then
			jlog_err ${f} "Section and Type cannot be specified!"
			return 6
		fi
	elif [ -n "${sec}" ] && [ -n "${typ}" ]; then
		jlog_err ${f} "Section and Type cannot be specified!"
		return 7
	else
		:
	fi
	return 0
}
_uci_get_load() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local keys=""
	local key
	local pkg sec typ opt
	local ld_rslt
	utils_check_retv "${retv}" "keys" "key" "pkg" "sec" "typ" "opt" \
		"ld_rslt" || return $?
	json_select "${UBUS_SIGN_IDX}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not load signature's JSON object!"
		return ${ret}
	fi
	json_get_keys keys
	for key in $keys; do
		case "${key}" in
		"config")
			json_get_var pkg "${key}"
			;;
		"section")
			json_get_var sec "${key}"
			;;
		"type")
			json_get_var typ "${key}"
			;;
		"option")
			json_get_var opt "${key}"
			;;
		*)
			jlog_err ${f} "Key '${key}' is invalid!"
			return 1
			;;
		esac
	done
	for tok in ${pkg} ${sec} ${typ} ${opt}; do
		if [ "${tok##*:}" != "${tok}" ]; then
			jlog_err ${f} "Token '${tok}' cannot have semicolons!"
			return 2
		fi
	done
	ld_rslt="${pkg}:${sec}:${typ}:${opt}"
	eval "${retv}=\${ld_rslt}"
	return 0
}
uci_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt=""
	local json
	utils_check_retv "${retv}" "rslt" "json" || return $?
	if [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_uci_get_load rslt || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error loading 'signature'!"
		return ${ret}
	fi
	_uci_get_validate "${rslt}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Request's 'signature' is not valid!"
		return ${ret}
	fi
	json_init
	_uci_get_reply "${rslt}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error building the signature's reply!"
		return ${ret}
	fi
	json=$(json_dump)
	json_cleanup
	eval "${retv}=\${json}"
	return 0
}
INC_UCI_GET=1
