#!/bin/sh
_jq_elem_typ_get() {
	local json="${1}"
	local name="${2}"
	local type="${3}"
	local outp="${4}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_query
	local jq_rslt
	local jq_type
	utils_check_retv "${outp}" "json" "name" "type" "f" "ret" "jq_query" \
		"jq_rslt" "jq_type" || return $?
	utils_cmd_exists "jq" || return $?
	jq_query='."'"${name}"'"'
	jq_rslt=$(jq "${jq_query}" 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	jq_query='type'
	jq_type=$(jq -r "${jq_query}" 2>&1 <<< "${jq_rslt}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_type}"
		return ${ret}
	fi
	if [ "${jq_type}" != "${type}" ]; then
		jlog_err ${f} "Element retrieved is not of '${type}' type!"
		jlog_err ${f} "${jq_rslt}"
		return 1
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_arr_idx_get() {
	local json="${1}"
	local name="${2}"
	local idx="${3}"
	local outp="${4}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_arr=""
	local jq_query
	local jq_rslt
	utils_check_retv "${outp}" "json" "name" "idx" "f" "ret" "jq_arr" \
		"jq_query" "jq_rslt" || return $?
	utils_cmd_exists "jq" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ] || [ -z "${idx}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	utils_is_num "${idx}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Argument 'idx' must be numeric!"
		return 1
	fi
	_jq_elem_typ_get "${json}" "${name}" "array" jq_arr || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting '${name}' element using 'jq'!"
		return ${ret}
	fi
	jq_query='.['"${idx}"']'
	jq_rslt=$(jq "${jq_query}" 2>&1 <<< "${jq_arr}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "'jq' command failed with '${ret}'"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_arr_len_get() {
	local json="${1}"
	local name="${2}"
	local outp="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_arr=""
	local jq_query
	local jq_rslt
	utils_check_retv "${outp}" "json" "name" "idx" "f" "ret" "jq_arr" \
		"jq_query" "jq_rslt" || return $?
	utils_cmd_exists "jq" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_jq_elem_typ_get "${json}" "${name}" "array" jq_arr || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting '${name}' element using 'jq'!"
		return ${ret}
	fi
	jq_query='length'
	jq_rslt=$(jq "${jq_query}" 2>&1 <<< "${jq_arr}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "'jq' command failed with '${ret}'"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_obj_get() {
	local json="${1}"
	local name="${2}"
	local outp="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_obj=""
	utils_check_retv "${outp}" "json" "name" "f" "ret" "jq_obj" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_jq_elem_typ_get "${json}" "${name}" "object" jq_obj || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting '${name}' element using 'jq'!"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_obj}'" || echo "${jq_obj}"
	return 0
}
jq_str_get() {
	local json="${1}"
	local name="${2}"
	local outp="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_str=""
	utils_check_retv "${outp}" "json" "name" "f" "ret" "jq_str" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	_jq_elem_typ_get "${json}" "${name}" "string" jq_str || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting '${name}' element using 'jq'!"
		return ${ret}
	fi
	jq_str=$(utils_rem_dquote "${jq_str}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not remove double quotes!"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_str}'" || echo "${jq_str}"
	return 0
}
jq_elem_get() {
	local json="${1}"
	local name="${2}"
	local outp="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_query
	local jq_rslt=""
	utils_check_retv "${outp}" "json" "name" "f" "ret" "jq_query" \
		"jq_elem" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	jq_query='."'"${name}"'"'
	jq_rslt=$(jq "${jq_query}" 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_keys_get() {
	local json="${1}"
	local outp="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_query
	local jq_rslt=""
	utils_check_retv "${outp}" "json" "f" "ret" "jq_query" "jq_rslt" \
		|| return $?
	utils_cmd_exists "jq" || return $?
	if [ -z "${json}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	jq_query='keys | .[]'
	jq_rslt=$(jq -r "${jq_query}" 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_types_get() {
	local json="${1}"
	local outp="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_query
	local jq_rslt=""
	utils_check_retv "${outp}" "json" "f" "ret" "jq_query" "jq_rslt" \
		|| return $?
	utils_cmd_exists "jq" || return $?
	if [ -z "${json}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	jq_query='map(type) | .[]'
	jq_rslt=$(jq -r "${jq_query}" 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_rslt}'" || echo "${jq_rslt}"
	return 0
}
jq_typ_get() {
	local json="${1}"
	local name="${2}"
	local outp="${3}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local jq_query
	local jq_rslt=""
	local jq_typ
	utils_check_retv "${outp}" "json" "name" "f" "ret" "jq_query" \
		"jq_rslt" || return $?
	utils_cmd_exists "jq" || return $?
	if [ -z "${json}" ] || [ -z "${name}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	jq_query='."'"${name}"'"'
	jq_rslt=$(jq "${jq_query}" 2>&1 <<< "${json}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_rslt}"
		return ${ret}
	fi
	jq_query='type'
	jq_typ=$(jq -r "${jq_query}" 2>&1 <<< "${jq_rslt}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error executing 'jq' command!"
		jlog_err ${f} "${jq_typ}"
		return ${ret}
	fi
	[ "${outp}" ] && eval ${outp}="'${jq_typ}'" || echo "${jq_typ}"
	return 0
}
