#!/bin/sh
VERSION_BTL="/barix/info/BOOTVERSION"
VERSION_KRNL="/proc/version"
VERSION_RFS="/etc/qiba-version"
VERSION_FW="/barix/info/VERSION"
VERSION_GBL="/etc/versions"
_version_conf_host() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local tmp="/tmp/instreamer-ice"
	VERSION_BTL="${tmp}/vrs_btl"
	VERSION_RFS="${tmp}/vrs_rfs"
	VERSION_FW="${tmp}/vrs_fw"
	VERSION_GBL="${tmp}/vrs_gbl"
	mkdir -p "${tmp}" &>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not create '"${tmp}"'!"
	fi
	slog_inf ${f} "Using '"${tmp}"' directory"
	return 0
}
_version_add_icecast() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	utils_cmd_exists "icecast" || return $?
	utils_cmd_exists "grep" || return $?
	utils_cmd_exists "awk" || return $?
	rslt=$(icecast --version 2>/dev/null | grep -w "^Icecast" 2>/dev/null \
		| awk '{print $2}' 2>/dev/null) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' getting Icecast version!"
		return ${ret}
	elif [ -z "${rslt}" ]; then
		jlog_err ${f} "Could not get Icecast version!"
		return 1
	fi
	json_add_string "icecast" "${rslt}"
	return 0
}
_version_add_audiostreamer() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	utils_cmd_exists "sed" || return $?
	rslt=$(sed -n 's/^audiostreamer=//p' "${VERSION_GBL}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error in 'sed' command!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	elif [ -z "${rslt}" ]; then
		jlog_err ${f} "No match found for 'audiostreamer' version!"
		return 1
	fi
	json_add_string "audiostreamer" "${rslt}"
	return 0
}
_version_add_firmware() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	read -r rslt < "${VERSION_FW}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'read' command!"
		return ${ret}
	fi
	json_add_string "firmware" "${rslt}"
	return 0
}
_version_add_rootfs() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	read -r rslt < "${VERSION_RFS}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'read' command!"
		return ${ret}
	fi
	json_add_string "rootfs" "${rslt}"
	return 0
}
_version_add_kernel() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	read -r rslt < "${VERSION_KRNL}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'read' command!"
		return ${ret}
	fi
	json_add_string "kernel" "${rslt}"
	return 0
}
_version_add_bootloader() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	read -r rslt < "${VERSION_BTL}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' executing 'read' command!"
		return ${ret}
	fi
	json_add_string "bootloader" "${rslt}"
	return 0
}
_version_add_backend() {
	local f="${FUNCNAME[0]}"
	local VERSION=$(opkg info instreamer-ice-webui-be | grep "^Version:" | awk '{print $2}')
	#	jlog_err ${f} "No version available!"
	json_add_string "backend" "${VERSION}"
	return 0
}
_version_add_frontend() {
	local f="${FUNCNAME[0]}"
	local VERSION=$(opkg info instreamer-ice-webui-fe | grep "^Version:" | awk '{print $2}')
	#	jlog_err ${f} "No version available!"
	json_add_string "frontend" "${VERSION}"
	return 0
}
_version_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local json
	utils_check_retv "${retv}" "json" || return $?
	json_init
	json_add_object "versions"
	_version_add_backend || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Backend version!"
	fi
	ret=0
	_version_add_frontend || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Frontend version!"
	fi
	ret=0	
	_version_add_bootloader || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Bootloader version!"
	fi
	ret=0
	_version_add_kernel || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Kernel version!"
	fi
	ret=0
	_version_add_rootfs || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Rootfs version!"
	fi
	ret=0
	_version_add_firmware || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Firmware version!"
	fi
	ret=0
	_version_add_audiostreamer || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Audiostreamer version!"
	fi
	ret=0
	_version_add_icecast || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' adding Icecast version!"
	fi
	json_close_object
	json=$(json_dump)
	json_cleanup
	eval "${retv}=\${json}"
	return 0
}
do_version() {
	local proc="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local vres=""
	utils_check_retv "${retv}" "proc" "vres" || return $?
	if [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	# TEST MODE! When testing the Backend Shell Service handler for Version
	if [ ${BE_TGT} -eq 0 ]; then
		_version_conf_host || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not configure host for testing!"
			return ${ret}
		fi
	fi
	case "${proc}" in
	get)
		_version_get vres || ret=$?
		;;
	*)
		jlog_err ${f} "Procedure '${proc}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error occurred in '${proc}' procedure!"
		return ${ret}
	fi
	eval "${retv}=\${vres}"
	return 0
}
INC_VERSION=1
