#!/bin/sh
# Script for updating audiostreamer. It is run by audiostreamer, perfroming a version check and running the update when new version is available.
# Ussage: ./update-script
#
set -e
 
scriptname=$(basename $0)
lock="/var/run/${scriptname}"

exec 200>$lock
flock -n 200 || exit 1
. tmp/up/date_vars

echo $PKG_URL
echo "Package version is $PKG_VER"

CURR_VER=`cat /barix/info/VERSION`

echo "Current version is $CURR_VER"

if [ "$PKG_VER" = "$CURR_VER" ]; then
        echo "No new version detected quiting"
        exit 0
fi

if grep -q mmcblk0p4 /etc/mtab ; then
	echo "SD partion found. Calling qiba-update-client"
        echo $PKG_URL > /mnt/data/update_servers.txt
	/etc/init.d/qiba-update-client restart
	exit 0
else
        logger -i -t "update-script" -p daemon.warn "No data partion on SD card found, going to download into /tmp! Stopping the main app to save RAM"

        WORKDIR=/tmp/updater-downloader/
        /etc/init.d/audiostreamer stop
        rm /var/log/messages*
fi

echo $WORKDIR

if [ -d $WORKDIR ]; then
	echo "$WORKDIR exists"
else
	mkdir $WORKDIR
fi
cd $WORKDIR
logger -i -t "update-script" -p daemon.info "Cleaning old packages from $WORKDIR"
for f in $WORKDIR/*.tar; do

    ## Check if the glob gets expanded to existing files.
    ## If not, f here will be exactly the pattern above
    ## and the exists test will evaluate to false.
    [ -e "$f" ] && rm *.tar || echo "files do not exist"

    ## This is all we needed to know, so we can break after the first iteration
    break
done

logger -i -t "update-script" -p daemon.info "Downloading $PKG_URL"
                                                                                                                              
/usr/bin/wget $PKG_URL
if [ "$?" -eq "0" ]; then
        logger -i -t "update-script" -p daemon.info "Download went OK. Updating now..."
	/etc/init.d/qiba-update-client stop
        /usr/bin/qiba-update-client -f *.tar
        if [ "$?" -eq "0" ]; then
                logger -i -t "update-script" -p daemon.info "Update went OK. Rebooting now..."
                /etc/init.d/audiostreamer stop
                sleep 1
                /sbin/reboot -f &
else
                logger -i -t "update-script" -p daemon.info "Update failed. Check package versions and integrity"
fi                                                                                                                     
	/etc/init.d/qiba-update-client start
else
# This will happen with broken connection or wrong URL in both case it makes sense that we restart main app to retry
        logger -i -t "update-script" -p daemon.err "Downloading of $PKG_URL failed."
fi

